package javax.naming.spi;

import javax.naming.NameParser;
import javax.naming.Binding;
import javax.naming.NameClassPair;
import javax.naming.NamingEnumeration;
import javax.naming.Name;
import javax.naming.NamingException;
import java.util.Hashtable;
import javax.naming.CannotProceedException;
import javax.naming.Context;

class ContinuationContext implements Context, Resolver
{
    protected CannotProceedException cpe;
    protected Hashtable<?, ?> env;
    protected Context contCtx;
    
    protected ContinuationContext(CannotProceedException cpe, Hashtable<?, ?> env) {
        this.contCtx = null;
        this.cpe = cpe;
        this.env = env;
    }
    
    protected Context getTargetContext() throws NamingException {
        if (this.contCtx == null) {
            if (this.cpe.getResolvedObj() == null) {
                throw (NamingException)this.cpe.fillInStackTrace();
            }
            this.contCtx = NamingManager.getContext(this.cpe.getResolvedObj(), this.cpe.getAltName(), this.cpe.getAltNameCtx(), this.env);
            if (this.contCtx == null) {
                throw (NamingException)this.cpe.fillInStackTrace();
            }
        }
        return this.contCtx;
    }
    
    @Override
    public Object lookup(Name name) throws NamingException {
        return this.getTargetContext().lookup(name);
    }
    
    @Override
    public Object lookup(String s) throws NamingException {
        return this.getTargetContext().lookup(s);
    }
    
    @Override
    public void bind(Name name, Object o) throws NamingException {
        this.getTargetContext().bind(name, o);
    }
    
    @Override
    public void bind(String s, Object o) throws NamingException {
        this.getTargetContext().bind(s, o);
    }
    
    @Override
    public void rebind(Name name, Object o) throws NamingException {
        this.getTargetContext().rebind(name, o);
    }
    
    @Override
    public void rebind(String s, Object o) throws NamingException {
        this.getTargetContext().rebind(s, o);
    }
    
    @Override
    public void unbind(Name name) throws NamingException {
        this.getTargetContext().unbind(name);
    }
    
    @Override
    public void unbind(String s) throws NamingException {
        this.getTargetContext().unbind(s);
    }
    
    @Override
    public void rename(Name name, Name name2) throws NamingException {
        this.getTargetContext().rename(name, name2);
    }
    
    @Override
    public void rename(String s, String s2) throws NamingException {
        this.getTargetContext().rename(s, s2);
    }
    
    @Override
    public NamingEnumeration<NameClassPair> list(Name name) throws NamingException {
        return this.getTargetContext().list(name);
    }
    
    @Override
    public NamingEnumeration<NameClassPair> list(String s) throws NamingException {
        return this.getTargetContext().list(s);
    }
    
    @Override
    public NamingEnumeration<Binding> listBindings(Name name) throws NamingException {
        return this.getTargetContext().listBindings(name);
    }
    
    @Override
    public NamingEnumeration<Binding> listBindings(String s) throws NamingException {
        return this.getTargetContext().listBindings(s);
    }
    
    @Override
    public void destroySubcontext(Name name) throws NamingException {
        this.getTargetContext().destroySubcontext(name);
    }
    
    @Override
    public void destroySubcontext(String s) throws NamingException {
        this.getTargetContext().destroySubcontext(s);
    }
    
    @Override
    public Context createSubcontext(Name name) throws NamingException {
        return this.getTargetContext().createSubcontext(name);
    }
    
    @Override
    public Context createSubcontext(String s) throws NamingException {
        return this.getTargetContext().createSubcontext(s);
    }
    
    @Override
    public Object lookupLink(Name name) throws NamingException {
        return this.getTargetContext().lookupLink(name);
    }
    
    @Override
    public Object lookupLink(String s) throws NamingException {
        return this.getTargetContext().lookupLink(s);
    }
    
    @Override
    public NameParser getNameParser(Name name) throws NamingException {
        return this.getTargetContext().getNameParser(name);
    }
    
    @Override
    public NameParser getNameParser(String s) throws NamingException {
        return this.getTargetContext().getNameParser(s);
    }
    
    @Override
    public Name composeName(Name name, Name name2) throws NamingException {
        return this.getTargetContext().composeName(name, name2);
    }
    
    @Override
    public String composeName(String s, String s2) throws NamingException {
        return this.getTargetContext().composeName(s, s2);
    }
    
    @Override
    public Object addToEnvironment(String s, Object o) throws NamingException {
        return this.getTargetContext().addToEnvironment(s, o);
    }
    
    @Override
    public Object removeFromEnvironment(String s) throws NamingException {
        return this.getTargetContext().removeFromEnvironment(s);
    }
    
    @Override
    public Hashtable<?, ?> getEnvironment() throws NamingException {
        return this.getTargetContext().getEnvironment();
    }
    
    @Override
    public String getNameInNamespace() throws NamingException {
        return this.getTargetContext().getNameInNamespace();
    }
    
    @Override
    public ResolveResult resolveToClass(Name name, Class<? extends Context> clazz) throws NamingException {
        if (this.cpe.getResolvedObj() == null) {
            throw (NamingException)this.cpe.fillInStackTrace();
        }
        Resolver resolver = NamingManager.getResolver(this.cpe.getResolvedObj(), this.cpe.getAltName(), this.cpe.getAltNameCtx(), this.env);
        if (resolver == null) {
            throw (NamingException)this.cpe.fillInStackTrace();
        }
        return resolver.resolveToClass(name, clazz);
    }
    
    @Override
    public ResolveResult resolveToClass(String s, Class<? extends Context> clazz) throws NamingException {
        if (this.cpe.getResolvedObj() == null) {
            throw (NamingException)this.cpe.fillInStackTrace();
        }
        Resolver resolver = NamingManager.getResolver(this.cpe.getResolvedObj(), this.cpe.getAltName(), this.cpe.getAltNameCtx(), this.env);
        if (resolver == null) {
            throw (NamingException)this.cpe.fillInStackTrace();
        }
        return resolver.resolveToClass(s, clazz);
    }
    
    @Override
    public void close() throws NamingException {
        this.cpe = null;
        this.env = null;
        if (this.contCtx != null) {
            this.contCtx.close();
            this.contCtx = null;
        }
    }
}
